/*******************************************************
 Author:					Manfred Langemann
 mailto:					Manfred.Langemann t t-online.de
 Begin of project:			02.05.2020
 Latest version generated:	02.05.2020
 Processor:					ATmega32
 Filename:					QMC5883L_Earth_Magnetic_Field_Sensor.c
 Description:				Main program for reading the QMC5883L Earth magnetic field sensor

 Used Timers:
	None
 Used Interrupts:
	None
 *******************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "General.h"
#include "Delay.h"
#include "RS232.h"
#include "QMC5883L.h"

int main (void)
	{
	uint8_t		i;
	int16_t		iEMF[3];
	float		xHeading, yHeading, zHeading;
	float		PI2;
/*
** Power-on delay of 0.1 second.
*/
	Delay_ms (100);
/*
** Init RS232 Interface with 9600 baud.
*/
	RS232_Init ();
/*
** Print Device Name and S/W Date and Time.
*/
	printf ("Hello, I am the QMC5883L Controller, ATmega32.\n");
	printf ("S/W Version: %s %s\n", __DATE__, __TIME__);
/*
** Initialise the QMC5883L sensor functionalities (incl. TWI interface).
*/
	if (!QMC5883L_Init ())
		{
		printf ("Cannot initialise QMS5883L Sensor via I2C line !\n");
		while (TRUE) {};
		}
/*
** Endless loop doing measurements.
*/
	while (TRUE)
		{
/*
** Do an Earth magnetic field measurement.
*/
		i = QMC5883L_ReadMagneticData (iEMF, TRUE);
		if (i != QMC5883L_OK) QMC5883L_ReportError (i);
/*
** Compute the angles of the Earth magnetic field.
*/
		PI2 = 2.0 * M_PI;

		xHeading = atan2 ((float)iEMF[1], (float)iEMF[0]);	// atan2 (Y,X)
		yHeading = atan2 ((float)iEMF[2], (float)iEMF[0]);	// atan2 (Z,X)
		zHeading = atan2 ((float)iEMF[2], (float)iEMF[1]);	// atan2 (Z,Y)
/*
** Convert the angles (-Pi to +Pi) to angles of 0 to 359 degree.
*/
		if (xHeading < 0)	xHeading = xHeading + PI2;
		if (xHeading > PI2)	xHeading = xHeading - PI2;
		if (yHeading < 0)	yHeading = yHeading + PI2;
		if (yHeading > PI2)	yHeading = yHeading - PI2;
		if (zHeading < 0)	zHeading = zHeading + PI2;
		if (zHeading > PI2)	zHeading = zHeading - PI2;
/*
** Convert angles from radians to degrees.
*/
		xHeading = xHeading * RAD2DEG;
		yHeading = yHeading * RAD2DEG;
		zHeading = zHeading * RAD2DEG;
/*
** Print angle values, we are happy with integer numbers.
*/		
		printf ("Heading = %i Deg, (%i, %i)\n",(int16_t)xHeading, (int16_t)yHeading, (int16_t)zHeading);
/*
** Delay of 0.5 second.
*/		
		Delay_ms (500);
		}
	}
