/*******************************************************
 Author:					Manfred Langemann
 mailto:					Manfred.Langemann t t-online.de
 Begin of project:			25.03.2007
 Latest version generated:	25.03.2007
 Filename:					RS232.c
 Description:    			Implementation of RS232 Interface

 Public Functions:


 Internal Functions:

Examples of "printf":

	uint8_t tmp;
	uint16_t UINT16;
	uint32_t UINT32;
	double	wert;

	tmp = 202;
	UINT16 = 2034;
	UINT32 = 1198654466L;
	wert = 1.3456e-3;

	printf ("tmp = %d\n", tmp);
	printf ("UINT16 = %d\n", UINT16);
	printf ("UINT32 = %ld\n", UINT32);

	printf ("tmp = 0x%02X\n", tmp);
	printf ("UINT16 = 0x%04X\n", UINT16);
	printf ("UINT32 = 0x%08lX\n", UINT32);

	printf ("Wert = %f\n", wert); ---> geht nicht
	printf ("Wert = %6.2f\n", wert);  ---> geht nicht
	--> in this case, use the library printflib

  Es geht auch printf of double Zahlen:
	double		a = 3.12345;
	char		s[20];
	dtostrf (a, 10, 5, s);

See page 55 of document avr-libc-user-manual-1.2.3.pdf
By default, all the conversions described above are available except the floating-point
conversions, and the [ conversion. These conversions will be available in the extended
version provided by the library libscanf_flt.a. Note that either of these conversions
requires the availability of a buffer that needs to be obtained at run-time using
malloc(). If this buffer cannot be obtained, the operation is aborted, returning the value
EOF. To link a program against the extended version, use the following compiler flags
in the link stage:
-Wl,-u,vfscanf -lscanf_flt -lm

A third version is available for environments that are tight on space. This version is
provided in the library libscanf_min.a, and can be requested using the following
options in the link stage:
-Wl,-u,vfscanf -lscanf_min -lm

Bezglich uart getchar habe ich folgendes in mikrocontroller.net gefunden:
Re: AVR Studio UART RS232 Ausgabe auf Hyperterminal

Die printf-Ausgabe erfolgt ber den festgelegten I/O-stream 'stdout'.
Den musst Du definieren und zwar so:

static FILE uart0_stream = FDEV_SETUP_STREAM(uart0_putchar,
uart0_getchar, _FDEV_SETUP_RW);

static FILE uart1_stream = FDEV_SETUP_STREAM(uart1_putchar,
uart1_getchar, _FDEV_SETUP_RW);

Damit habe ich zwei Files definiert uart0_stream und uart1_stream.
Diese werden nun (GANZ wichtig!) dem Standardstream 'stdout' zugeordnet,
z.B.:

stdout = &uart0_stream;
//stdout = &uart1_stream;

..also so gehts bei mir. Vielleicht hilfts..?

Evtl. liegts aber wirklich am Terminalprogramm?
Teraterm hier ist nicht ganz so seltsam und bockig wie Hyperterm:
http://hp.vector.co.jp/authors/VA002416/teraterm.html
Man kann Einstellungen speichern und es hat auch eine "Reset
Port"-Funktion, mit der man den UART des PC wieder flottmachen kann,
wenn er sich durch undefiniertes Portgeklimper abgeseilt hat...

Grle
 *******************************************************/
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "General.h"
#include "RS232.h"
/*
** Private Functions
** None
*/
/*
** Baud Rate fr die Serielle Schnittstelle
*/
#define BAUD_RATE		9600
/*
** Where the byte via UART Interrupt will be saved
** Use 'volatile', otherwise, data is lost
*/
volatile uint8_t	UART_Byte;
volatile uint8_t	UART_Byte_Status;
/*
** Define a stream for the printf function, using the UART
*/
FILE uart_str = FDEV_SETUP_STREAM(RS232_putchar, NULL, _FDEV_SETUP_RW);

/*******************************************************
 Function: RS232_Init

 Purpose: Initialise the RS232 serial link

 Input Parameter: None

 Return Value: None

*******************************************************/
void RS232_Init (void)
	{
/*
** Enable TXEN and RXEN in register UCSRB
*/
	UCSRB |= (1 << TXEN);
//	UCSRB |= (1 << RXEN);
/*
** Enable Interrupt Receive, if needed
*/
//	UCSRB |= (1 << RXCIE);
/*
** Set transmission type, Asynchron 8N1
*/
	UCSRC |= (1 << URSEL)|(3<<UCSZ0);
/*
** Set baud rate
*/
    UBRRH = (uint8_t) ((SYSCLOCK / (BAUD_RATE * 16L) - 1)>>8);
    UBRRL = (uint8_t) (SYSCLOCK / (BAUD_RATE * 16L) - 1);
/*
** Open a channel for printf
*/
	stdout = &uart_str;
//	fdevopen (RS232_putchar, NULL);
/*
** Set the UART CMD bytes
*/
	UART_Byte = 0;
	UART_Byte_Status = 0;
	}
/*******************************************************
 Function: RS232_putchar

 Purpose: Initialise function "printf "

 Input Parameter: char buffer

 Return Value:
 	Value = 0

*******************************************************/
int RS232_putchar (char c, FILE *stream)
	{
/*
** Check on NEWLINE
*/
	if (c == '\n')
  		{
		RS232_putchar ('\r', stream);
		}
/*
** Wait until previous character is sent
*/  
	loop_until_bit_is_set(UCSRA, UDRE);
/*
** Send character to UART
*/
	UDR = c;

	return 0;
	}
/*******************************************************
 Function: RS232_String

 Purpose: Send to the UART an ASCII string

 Input Parameter: char string

 Return Value: None
 	Value = 0

*******************************************************/
void RS232_PutString (const char* s)
	{
	do
		{
/*
** Check on NEWLINE, if yes, then send first a carriage return.
*/
		if (*s == '\n')
			{
			RS232_PutByte (13);
			}
/*
** Ein Zeichen des strings senden.
*/
		RS232_PutByte ((uint8_t)*s);
/*
** So lange bis der String zu ende ist.
*/
		s++;
		} while (*s != 0);
	
	return;
	}
/*******************************************************
 Function: RS232_PutByte

 Purpose: Send to the UART a data byte

 Input Parameter: the byte

 Return Value: None
 	Value = 0

*******************************************************/
void RS232_PutByte (uint8_t Byte)
	{
/*
** Wait until previous character is sent
*/
	while (!(UCSRA & (1<<UDRE)));
/*
** Send byte to UART
*/
	UDR = Byte;

	return;
	}
/*******************************************************
 Function: RS232_Put_unit16

 Purpose: Send a two byte word (uin16_t) to the UART, LSB first

 Input Parameter: Address of uint16_t

 Return Value: None

*******************************************************/
void RS232_Put_unit16 (uint16_t *Word)
	{
	uint8_t 	i;
	uint8_t		*py;

	py = (uint8_t*) Word;
	for (i=0;i<2;i++)
		{
		RS232_PutByte (*py++);
		}

	return;
	}
/*******************************************************
 Function: RS232_Put_unit32

 Purpose: Send a four byte word (uin32_t) to the UART, LSB first

 Input Parameter:  Address of uint32_t

 Return Value: None
 
*******************************************************/
void RS232_Put_unit32 (uint32_t *DWord)
	{
	uint8_t 	i;
	uint8_t		*py;

	py = (uint8_t*) DWord;

	for (i=0;i<4;i++)
		{
		RS232_PutByte (*py++);
		}

	return;
	}
/*******************************************************
 Function: RS232_GetByte

 Purpose: Get from the UART a byte

 Input Parameter: None

 Return Value: The UART byte

*******************************************************/
uint8_t RS232_GetByte (void)
	{
/*
** Wait until byte is available
*/
	while (!(UCSRA &(1<<RXC)));
/*
** Get and return the byte from UART
*/	
	return UDR;
	}
/*******************************************************
 Function: Interrupt Vector Routine UART RX complete

 Purpose:
	Will be called by interrupt,
	when a byte has been received by the UART
	
 *******************************************************/
ISR(USART_RXC_vect)
	{
/*
** Disable all interrupts
*/
	cli ();
/*
** Get the byte from the UART
*/
	UART_Byte = UDR;
	UART_Byte_Status = 1;
/*
** Enable all interrupts
*/
	sei ();
	}
